// Program to time the linear search algorithm

#include <iostream>
#include <ctime>
using namespace std;

// Prototypes
int linearSearchList(int list[], int numElems, int value);
int linearSearchSortedList(int list[], int numElems, int value);

const int ARRAY_SIZE = 100000;

int main() {
	int evenList[ARRAY_SIZE];
	int evenNumber, oddNumber, index, location;
	unsigned startTime, endTime;

	// fill evenList with even values 0, 2, 4, ..., 199996, 199998
	evenNumber = 0;
	for (index = 0; index < ARRAY_SIZE; index++) {
		evenList[index] = evenNumber;
		evenNumber += 2;
	} // end for

	// Time successful searches for target values 0, 2, 4, ..., 199998
	startTime = static_cast<unsigned>(time(0));
	for (evenNumber = 0; evenNumber <= ARRAY_SIZE*2-2; evenNumber += 2) {
		location = linearSearchList(evenList, ARRAY_SIZE, evenNumber);
	} // end for
	endTime = static_cast<unsigned>(time(0));
	cout << "linearSearchList of " << ARRAY_SIZE << " successful searches "
		<< "took " << endTime - startTime << " seconds." << endl;	
	
	// Time unsuccessful searches for target values 1, 3, 5, ..., 199999
	startTime = static_cast<unsigned>(time(0));
	for (oddNumber = 1; oddNumber <= ARRAY_SIZE*2-1; oddNumber += 2) {
		location = linearSearchList(evenList, ARRAY_SIZE, oddNumber);
	} // end for
	endTime = static_cast<unsigned>(time(0));
	cout << "linearSearchList of " << ARRAY_SIZE << " unsuccessful searches "
		<< "took " << endTime - startTime << " seconds." << endl;	

	// Time successful searches for target values 0, 2, 4, ..., 199998
	startTime = static_cast<unsigned>(time(0));
	for (evenNumber = 0; evenNumber <= ARRAY_SIZE*2-2; evenNumber += 2) {
		location = linearSearchSortedList(evenList, ARRAY_SIZE, evenNumber);
	} // end for
	endTime = static_cast<unsigned>(time(0));
	cout << "linearSearchSortedList of " << ARRAY_SIZE << " successful searches "
		<< "took " << endTime - startTime << " seconds." << endl;	
	
	// Time unsuccessful searches for target values 1, 3, 5, ..., 199999
	startTime = static_cast<unsigned>(time(0));
	for (oddNumber = 1; oddNumber <= ARRAY_SIZE*2-1; oddNumber += 2) {
		location = linearSearchSortedList(evenList, ARRAY_SIZE, oddNumber);
	} // end for
	endTime = static_cast<unsigned>(time(0));
	cout << "linearSearchSortedList of " << ARRAY_SIZE << " unsuccessful searches "
		<< "took " << endTime - startTime << " seconds." << endl;	

} // end main


//*****************************************************************
// The linearSearchList function performs a linear search on an   *
// integer array. The array list, which has a maximum of numElems *
// elements, is searched for the number stored in value. If the   *
// number is found, its array subscript is returned. Otherwise,   *
// -1 is returned indicating the value was not in the array.      *
//*****************************************************************

int linearSearchList(int list[], int numElems, int value) {
   int index = 0;       // Used as a subscript to search array
   int position = -1;   // To record position of search value
   bool found = false;  // Flag to indicate if the value was found

   while (index < numElems && !found) {

      if (list[index] == value) { // If the value is found
         found = true;            // Set the flag
         position = index;        // Record the value's subscript
      } // end if

      index++;                    // Go to the next element
   } // end while

   return position;               // Return the position, or -1
} // end linearSearchList


//*****************************************************************
// The linearSearchSortedList function performs a linear search on a sorted *
// integer array. The array list, which has a maximum of numElems *
// elements, is searched for the number stored in value. If the   *
// number is found, its array subscript is returned. Otherwise,   *
// -1 is returned indicating the value was not in the array.      *
//*****************************************************************

int linearSearchSortedList(int list[], int numElems, int value) {
	int index = 0;       // Used as a subscript to search array

	while (index < numElems) {
		
		if (list[index] > value) {
			return -1;
		} else if (list[index] == value) { // If the value is found
			return index;			// return the value's subscript
		} // end if

		index++;                    // Go to the next element
	} // end while

	return -1;               
} // end linearSearchSortedList